/*
 * This Source Code is subject to the terms of the Mozilla Public License
 * version 2.0 (the "License"). You can obtain a copy of the License at
 * http://mozilla.org/MPL/2.0/.
 *
 * The Original Code is Forward Extension.
 * The Initial Developer of the Original Code is Onno Ekker.
 * Portions created by Onno Ekker <o.e.ekker@gmail.com> are Copyright
 * (c) 2008. All Rights Reserved.
 */

"use strict";

(function() { 

const THUNDERBIRD_ID = "{3550f703-e582-4d05-9a08-453d09bdfdc6}";

window.myForwardExtension = {

  myLogToErrorConsole: function(aMessage, aSourceName, aSourceLine, aLineNumber,
                                aColumnNumber, aFlags, aCategory)
  {
    var consoleService = Components.classes["@mozilla.org/consoleservice;1"].
                                    getService(Components.interfaces.nsIConsoleService);
    if (aFlags === null)
    {
      consoleService.logStringMessage(aMessage);
    }
    else
    {
      var scriptError = Components.classes["@mozilla.org/scripterror;1"].
                                   createInstance(Components.interfaces.nsIScriptError);
      scriptError.init(aMessage, aSourceName, aSourceLine, aLineNumber,
                       aColumnNumber, aFlags, aCategory);
      consoleService.logMessage(scriptError);
    }
  },

  // forward-button

  updateForwardButtonMenu: function(event)
  {
    var prefService = Components.classes["@mozilla.org/preferences-service;1"].
                                 getService(Components.interfaces.nsIPrefService);
    var forwardType = 2; // Make Inline the default for forwarding, in case preference cannot be retrieved
    var el;

    // mail.forward_message_mode could be 1, if the user migrated from Netscape
    // Messenger 4.x
    // 1 (forward as quoted) is obsolete, so we treat it as forward inline
    // since that is more like forward as quoted then forward as attachment
    if (prefService.getPrefType("mail.forward_message_mode") === prefService.PREF_INT) {
      try
      {
        forwardType = prefService.getIntPref("mail.forward_message_mode");
      }
      catch (ex)
      {
        myForwardExtension.myLogToErrorConsole(ex, "chrome://forward/content/forward.js",
                                               "      forwardType = nsIPrefBranch.getIntPref(\"mail.forward_message_mode\");",
                                               55, 0, 9, "javascript");
      }
    }

    el = document.getElementById("button-forwardAsMenu-forwardInline");
    if (el !== null)
    {
      el.setAttribute("default", (forwardType === 2));
    }
    el = document.getElementById("button-forwardAsMenu-forwardAsAttachment");
    if (el !== null)
    {
      el.setAttribute("default", (forwardType !== 2));
    }

    el = document.getElementById("hdrForwardButton");  // Thunderbird 3.0pre+
    if (el === null)
    {
      var headerViewButtonBox = document.getElementById("msgHeaderViewDeck").selectedPanel.
                                         getElementsByTagName("header-view-button-box").item(0);  // Thunderbird 3.0a1 .. 3.0b4
      if (headerViewButtonBox !== null)
      {
        el = document.getAnonymousElementByAttribute(headerViewButtonBox, "anonid", "hdrForwardButton");
      }
    }
    if (el !== null)
    {
      var headerForwardButtonFirstInline = document.getElementById("hdrForwardButtonFirstInline");
      var headerForwardButtonLastInline = document.getElementById("hdrForwardButtonLastInline");
      if (headerForwardButtonFirstInline !== null) // Dropdown menuitems already exist, update if pref change
      {
        if (forwardType !== 2)
        {
          if (headerForwardButtonLastInline.hasAttribute("hidden")) // Show as attachment as first item
          {
            headerForwardButtonFirstInline.setAttribute("hidden", "true"); // by hiding first and showing last
            headerForwardButtonLastInline.removeAttribute("hidden");
          }
        }
        else
        {
          if (headerForwardButtonFirstInline.hasAttribute("hidden")) // Show inline as first item
          {
            headerForwardButtonFirstInline.removeAttribute("hidden");  // by showing first and hiding last
            headerForwardButtonLastInline.setAttribute("hidden", "true");
          }
        }
      }
      else
      {
        var strbundle = document.getElementById("bundle_forward");
        var osString = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULRuntime).OS;
        var appInfo = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);
        var versionChecker = Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator);
        var headerInlineLabel, headerAsAttachmentLabel, headerButtonTooltip;
        var image = window.getComputedStyle(el, null).getPropertyValue("list-style-image");
        var region = window.getComputedStyle(el, null).getPropertyValue("-moz-image-region");
        var menupopup = el.getElementsByTagName("menupopup").item(0);
        var menuitem;
        el.setAttribute("type", "menu-button");
        if ((osString !== "WINNT") && (osString !== "Linux") && (osString !== "Darwin"))
        {
          el.setAttribute("class", "msgHeaderView-button hdrReplyButton"); // Make forward button look like reply button
          el.setAttribute("style", "list-style-image: " + image + "; -moz-image-region: " + region + ";"); // except for the icon
        }

        if (menupopup === null)
        {
          el.appendChild(document.createElement("menupopup"));
          menupopup = el.firstChild;
        }

        if (appInfo.ID === THUNDERBIRD_ID && versionChecker.compare(appInfo.version, "9.0") >= 0)
        {
          // Running Thunderbird 9.0+ (see bug 478468)
          headerInlineLabel = strbundle.getString("header.inline.label1");
          headerAsAttachmentLabel = strbundle.getString("header.asAttachment.label1");
          headerButtonTooltip = strbundle.getString("hdrForwardButton.tooltip1");
        }
        else
        {
          headerInlineLabel = strbundle.getString("header.inline.label");
          headerAsAttachmentLabel = strbundle.getString("header.asAttachment.label");
          headerButtonTooltip = strbundle.getString("hdrForwardButton.tooltip");
        }

        menuitem = menupopup.appendChild(document.createElement("menuitem"));
        menuitem.setAttribute("id", "hdrForwardButtonFirstInline");
        menuitem.setAttribute("label", headerInlineLabel);
        menuitem.setAttribute("tooltiptext", headerButtonTooltip);
        menuitem.addEventListener("command", function(event) { MsgForwardAsInline(event); event.stopPropagation(); RestoreFocusAfterHdrButton(); }, false);
        if (forwardType !== 2) { menuitem.setAttribute("hidden", "true"); }  // Show first depending on pref

        menuitem = menupopup.appendChild(document.createElement("menuitem"));
        menuitem.setAttribute("id", "hdrForwardButtonAsAttachment");
        menuitem.setAttribute("label", headerAsAttachmentLabel);
        menuitem.setAttribute("tooltiptext", headerButtonTooltip);
        menuitem.addEventListener("command", function(event) { MsgForwardAsAttachment(event); event.stopPropagation(); RestoreFocusAfterHdrButton(); }, false);

        menuitem = menupopup.appendChild(document.createElement("menuitem"));
        menuitem.setAttribute("id", "hdrForwardButtonLastInline");
        menuitem.setAttribute("label", headerInlineLabel);
        menuitem.setAttribute("tooltiptext", headerButtonTooltip);
        menuitem.addEventListener("command", function(event) { MsgForwardAsInline(event); event.stopPropagation(); RestoreFocusAfterHdrButton(); }, false);
        if (forwardType === 2) { menuitem.setAttribute("hidden", "true"); }  // Show last depending on pref
      }
    }

    // The CompactHeader extension hides the hdrForwardButton and adds a copy of
    // the button-forward button from the Mail toolbar to the msgHeaderViewDeck
    el = document.getElementById("msgHeaderViewDeck").getElementsByClassName("customize-header-toolbar-button-forward").item(0);
    if (el !== null) {
      var headerForwardButtonFirstInline1 = document.getElementById("hdrForwardButtonFirstInline1");
      var headerForwardButtonLastInline1 = document.getElementById("hdrForwardButtonLastInline1");
      if (headerForwardButtonFirstInline1 !== null) // Dropdown menuitems already exist, update if pref change
      {
        if (forwardType !== 2)
        {
          if (headerForwardButtonLastInline1.hasAttribute("hidden")) // Show as attachment as first item
          {
            headerForwardButtonFirstInline1.setAttribute("hidden", "true"); // by hiding first and showing last
            headerForwardButtonLastInline1.removeAttribute("hidden");
          }
        }
        else
        {
          if (headerForwardButtonFirstInline1.hasAttribute("hidden")) // Show inline as first item
          {
            headerForwardButtonFirstInline1.removeAttribute("hidden");  // by showing first and hiding last
            headerForwardButtonLastInline1.setAttribute("hidden", "true");
          }
        }
      }
      else
      {
        var strbundle = document.getElementById("bundle_forward");
        var appInfo = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);
        var versionChecker = Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator);
        var headerInlineLabel, headerAsAttachmentLabel, headerButtonTooltip;
        var menupopup = el.getElementsByTagName("menupopup").item(0);
        var menuitem;

        menuitem = menupopup.removeChild(menupopup.firstChild);
        menuitem = menupopup.removeChild(menupopup.firstChild);

        if (appInfo.ID === THUNDERBIRD_ID && versionChecker.compare(appInfo.version, "9.0") >= 0)
        {
          // Running Thunderbird 9.0+ (see bug 478468)
          headerInlineLabel = strbundle.getString("header.inline.label1");
          headerAsAttachmentLabel = strbundle.getString("header.asAttachment.label1");
          headerButtonTooltip = strbundle.getString("hdrForwardButton.tooltip1");
        }
        else
        {
          headerInlineLabel = strbundle.getString("header.inline.label");
          headerAsAttachmentLabel = strbundle.getString("header.asAttachment.label");
          headerButtonTooltip = strbundle.getString("hdrForwardButton.tooltip");
        }

        menuitem = menupopup.appendChild(document.createElement("menuitem"));
        menuitem.setAttribute("id", "hdrForwardButtonFirstInline1");
        menuitem.setAttribute("label", headerInlineLabel);
        menuitem.setAttribute("tooltiptext", headerButtonTooltip);
        menuitem.addEventListener("command", function(event) { MsgForwardAsInline(event); event.stopPropagation(); RestoreFocusAfterHdrButton(); }, false);
        if (forwardType !== 2) { menuitem.setAttribute("hidden", "true"); }  // Show first depending on pref

        menuitem = menupopup.appendChild(document.createElement("menuitem"));
        menuitem.setAttribute("id", "hdrForwardButtonAsAttachment1");
        menuitem.setAttribute("label", headerAsAttachmentLabel);
        menuitem.setAttribute("tooltiptext", headerButtonTooltip);
        menuitem.addEventListener("command", function(event) { MsgForwardAsAttachment(event); event.stopPropagation(); RestoreFocusAfterHdrButton(); }, false);

        menuitem = menupopup.appendChild(document.createElement("menuitem"));
        menuitem.setAttribute("id", "hdrForwardButtonLastInline1");
        menuitem.setAttribute("label", headerInlineLabel);
        menuitem.setAttribute("tooltiptext", headerButtonTooltip);
        menuitem.addEventListener("command", function(event) { MsgForwardAsInline(event); event.stopPropagation(); RestoreFocusAfterHdrButton(); }, false);
        if (forwardType === 2) { menuitem.setAttribute("hidden", "true"); }  // Show last depending on pref
      }
    }
  },

  doCommandForwardButton: function (event)
  {
    var numSelected = GetNumSelectedMessages();
    if (numSelected === 1)
      MsgForwardMessage(event);
    else if (numSelected > 1)
      MsgForwardAsAttachment(event);
    return false;
  },

  // Initialize optional localized strings

  initOptionalStrings: function ()
  {
    var appInfo = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);
    var versionChecker = Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator);
    var strbundle = document.getElementById("bundle_forward");
    var el;

    function ucfirst(menuItem)
    {
      var origLabel = menuItem.getAttribute("label");
      var newLabel = origLabel.replace(/^(\S)/, function(t, a) { return a.toLocaleUpperCase(); });
      if (newLabel !== origLabel)
      {
        menuItem.setAttribute("label", newLabel);
      }
    }

    el = document.getElementById("button-forwardAsMenu-forwardInline");
    if (el !== null)
    {
      try
      {
        el.setAttribute("label", strbundle.getString("toolbar.inline.label"));
      }
      catch (ex)
      {
        ucfirst(el);
      }
    }
    el = document.getElementById("button-forwardAsMenu-forwardAsAttachment");
    if (el !== null)
    {
      try
      {
        el.setAttribute("label", strbundle.getString("toolbar.asAttachment.label"));
      }
      catch (ex)
      {
        ucfirst(el);
      }
    }

    el = document.getElementById("appmenu_forwardAsAttachments");
    if (el !== null)
    {
      try
      {
        el.setAttribute("label", strbundle.getString("forwardAsAttachments.label"));
      }
      catch (ex)
      {
      }
    }

    el = document.getElementById("menu_forwardAsAttachments");
    if (el !== null)
    {
      try
      {
        el.setAttribute("label", strbundle.getString("forwardAsAttachments.label"));
        el.setAttribute("accesskey", strbundle.getString("forwardAsAttachments.accesskey"));
      }
      catch (ex)
      {
      }
    }

    // if Thunderbird 3.0+
    el = document.getElementById("mailContext-forwardAsMenu");
    if (el !== null)
    {
      try
      {
        el.setAttribute("label", strbundle.getString("forwardAsMenu.label"));
        el.setAttribute("accesskey", strbundle.getString("forwardAsMenu.accesskey"));
      }
      catch (ex)
      {
      }
    }
    el = document.getElementById("mailContext-forwardAsMenu-forwardAsInline");
    if (el !== null)
    {
      try
      {
        el.setAttribute("label", strbundle.getString("forwardAsInline.label"));
        el.setAttribute("accesskey", strbundle.getString("forwardAsInline.accesskey"));
      }
      catch (ex)
      {
      }
    }
    el = document.getElementById("mailContext-forwardAsMenu-forwardAsAttachment");
    if (el !== null)
    {
      try
      {
        el.setAttribute("label", strbundle.getString("forwardAsAttachment.label"));
        el.setAttribute("accesskey", strbundle.getString("forwardAsAttachment.accesskey"));
      }
      catch (ex)
      {
      }
    }

    // else if Thunderbird 1.5..2.x
    el = document.getElementById("threadPaneContext-forwardAsMenu");
    if (el !== null)
    {
      try
      {
        el.setAttribute("label", strbundle.getString("forwardAsMenu.label"));
        el.setAttribute("accesskey", strbundle.getString("forwardAsMenu.accesskey"));
      }
      catch (ex)
      {
      }
    }
    el = document.getElementById("threadPaneContext-forwardAsMenu-forwardAsInline");
    if (el !== null)
    {
      try
      {
        el.setAttribute("label", strbundle.getString("forwardAsInline.label"));
        el.setAttribute("accesskey", strbundle.getString("forwardAsInline.accesskey"));
      }
      catch (ex)
      {
      }
    }
    el = document.getElementById("threadPaneContext-forwardAsMenu-forwardAsAttachment");
    if (el !== null)
    {
      try
      {
        el.setAttribute("label", strbundle.getString("forwardAsAttachment.label"));
        el.setAttribute("accesskey", strbundle.getString("forwardAsAttachment.accesskey"));
      }
      catch (ex)
      {
      }
    }
    el = document.getElementById("messagePaneContext-forwardAsMenu");
    if (el !== null)
    {
      try
      {
        el.setAttribute("label", strbundle.getString("forwardAsMenu.label"));
        el.setAttribute("accesskey", strbundle.getString("forwardAsMenu.accesskey"));
      }
      catch (ex)
      {
      }
    }
    el = document.getElementById("messagePaneContext-forwardAsMenu-forwardAsInline");
    if (el !== null)
    {
      try
      {
        el.setAttribute("label", strbundle.getString("forwardAsInline.label"));
        el.setAttribute("accesskey", strbundle.getString("forwardAsInline.accesskey"));
      }
      catch (ex)
      {
      }
    }
    el = document.getElementById("messagePaneContext-forwardAsMenu-forwardAsAttachment");
    if (el !== null)
    {
      try
      {
        el.setAttribute("label", strbundle.getString("forwardAsAttachment.label"));
        el.setAttribute("accesskey", strbundle.getString("forwardAsAttachment.accesskey"));
      }
      catch (ex)
      {
      }
    }

    if (appInfo.ID === THUNDERBIRD_ID && versionChecker.compare(appInfo.version, "24.0a1") >= 0)
    {
      // Hide new Thunderbird 24.0 menuitems
      el = document.getElementById("button-ForwardAsInlineMenu");
      if (el !== null)
        {
        try
        {
          el.setAttribute("hidden", true);
        }
        catch (ex)
        {
        }
      }
      el = document.getElementById("button-ForwardAsAttachmentMenu");
      if (el !== null)
        {
        try
        {
          el.setAttribute("hidden", true);
        }
        catch (ex)
        {
        }
      }
      el = document.getElementById("mailContext-forwardAsMenu-forwardAsInline");
      if (el !== null)
        {
        try
        {
          el.setAttribute("hidden", true);
        }
        catch (ex)
        {
        }
      }
      el = document.getElementById("mailContext-forwardAsMenu-forwardAsAttachment");
      if (el !== null)
        {
        try
        {
          el.setAttribute("hidden", true);
        }
        catch (ex)
        {
        }
      }
    }
  },

  // mail, threadPane and messagePane menu menuitem

  SetupForwardAsMenuMenuItem: function (menuID, numSelected, forceHide)
  {
    ShowMenuItem(menuID, (numSelected <= 1) && !forceHide);
    EnableMenuItem(menuID, (numSelected === 1));
  },

  // if Thunderbird 3.0+
  forward_fillMailContextMenu: function ()
  {
    var numSelected = GetNumSelectedMessages();

    // don't show mail items for links/images, just show related items.
    var hideMailItems = gContextMenu.onImage || gContextMenu.onLink;
    myForwardExtension.SetupForwardAsMenuMenuItem("mailContext-forwardAsMenu", numSelected, (numSelected === 0 || hideMailItems));
  },

  // else if Thunderbird 1.5..2.x
  forward_fillThreadPaneContextMenu: function ()
  {
    var numSelected = GetNumSelectedMessages();
    myForwardExtension.SetupForwardAsMenuMenuItem("threadPaneContext-forwardAsMenu", numSelected, false);
  },
  forward_fillMessagePaneContextMenu: function ()
  {
    var message = GetLoadedMessage();
    var numSelected = (message) ? 1 : 0;

    // don't show mail items for links/images, just show related items.
    var hideMailItems = gContextMenu.onImage || gContextMenu.onLink;
    myForwardExtension.SetupForwardAsMenuMenuItem("messagePaneContext-forwardAsMenu", numSelected, (numSelected === 0 || hideMailItems));
  },

  selectionChanged: function (event)
  {
    goUpdateCommand("cmd_forwardAsAttachments");
  },

  // Command controller

  forwardController: {
    supportsCommand : function(command)
    {
      switch(command) {
        case "cmd_forwardAsAttachments":
          return true;
        default:
          return false;
      }
    },
    isCommandEnabled: function(command)
    {
      switch(command) {
        case "cmd_forwardAsAttachments":
          var windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService().QueryInterface(Components.interfaces.nsIWindowMediator);
          var mail3paneWindow = windowMediator.getMostRecentWindow("mail:3pane");
          if (mail3paneWindow !== null) {
            var appmenu_forwardAsAttachments = document.getElementById("appmenu_forwardAsAttachments");
            var appmenu_forwardMsg = document.getElementById("appmenu_forwardMsg");
            var menu_forwardAsAttachments = document.getElementById("menu_forwardAsAttachments");
            var menu_forwardMsg = document.getElementById("menu_forwardMsg");
            if (GetNumSelectedMessages() > 1) {
              if (appmenu_forwardAsAttachments !== null) {
                appmenu_forwardAsAttachments.hidden = false;
                if (appmenu_forwardMsg.hasAttribute("key")) {
                  appmenu_forwardMsg.removeAttribute("key");
                  appmenu_forwardAsAttachments.setAttribute("key", "key_forward");
                }
                if (menu_forwardAsAttachments !== null) {
                  menu_forwardAsAttachments.hidden = false;
                  if (menu_forwardMsg.hasAttribute("key")) {
                    menu_forwardMsg.removeAttribute("key");
                    menu_forwardAsAttachments.setAttribute("key", "key_forward");
                  }
                }
              }
              return true;
            } else {
              if (appmenu_forwardAsAttachments !== null) {
                appmenu_forwardAsAttachments.hidden = true;
                if (appmenu_forwardAsAttachments.hasAttribute("key")) {
                  appmenu_forwardAsAttachments.removeAttribute("key");
                  appmenu_forwardMsg.setAttribute("key", "key_forward");
                }
                if (menu_forwardAsAttachments !== null) {
                  menu_forwardAsAttachments.hidden = true;
                  if (menu_forwardAsAttachments.hasAttribute("key")) {
                    menu_forwardAsAttachments.removeAttribute("key");
                    menu_forwardMsg.setAttribute("key", "key_forward");
                  }
                }
//                if (key_forward !== null)
//                  key_forward.setAttribute("oncommand", "goDoCommand('cmd_forward')");
              }
              return false;
            }
          }
          return false;
        default:
          return false;
      }
    },
    doCommand: function(command)
    {
      switch(command) {
        case "cmd_forwardAsAttachments":
          // if the user invoked a key short cut then it is possible that we got here for a command which is
          // really disabled. kick out if the command should be disabled.
          if (!this.isCommandEnabled(command)) return;
          MsgForwardAsAttachment();
          break;
      }
    }
  },

  // Event listeners

  installListeners: function (event)
  {
    var el;

    myForwardExtension.initOptionalStrings();
    myForwardExtension.updateForwardButtonMenu();

    el = document.getElementById("button-forward");
    if (el !== null)
    {
      // Thunderbird < 24.0 used oncommand instead of command on the button-forward 
      if (el.hasAttribute("oncommand")) {
        el.removeAttribute("oncommand");
        el.addEventListener("command", function(event) { myForwardExtension.doCommandForwardButton(event); event.stopPropagation(); }, false);
      }
    }

    el = document.getElementById("threadTree");
    if (el !== null)
    {
      el.addEventListener("select", myForwardExtension.selectionChanged, false);
    }

    // if Thunderbird 3.0+
    el = document.getElementById("mailContext");
    if (el !== null)
    {
      el.addEventListener("popupshowing", myForwardExtension.forward_fillMailContextMenu, false);
    }

    // else if Thunderbird 1.5..2.x
    el = document.getElementById("threadPaneContext");
    if (el !== null)
    {
      el.addEventListener("popupshowing", myForwardExtension.forward_fillThreadPaneContextMenu, false);
    }

    el = document.getElementById("messagePaneContext");
    if (el !== null)
    {
      el.addEventListener("popupshowing", myForwardExtension.forward_fillMessagePaneContextMenu, false);
    }
  },

  uninstallListeners: function (event)
  {
    var el;

    el = document.getElementById("threadTree");
    if (el !== null)
    {
      el.removeEventListener("select", myForwardExtension.selectionChanged, false);
    }

    // if Thunderbird 3.0+
    el = document.getElementById("mailContext");
    if (el !== null)
    {
      el.removeEventListener("popupshowing", myForwardExtension.forward_fillMailContextMenu, false);
    }

    // else if Thunderbird 1.5..2.x
    el = document.getElementById("threadPaneContext");
    if (el !== null)
    {
      el.removeEventListener("popupshowing", myForwardExtension.forward_fillThreadPaneContextMenu, false);
    }
    el = document.getElementById("messagePaneContext");
    if (el !== null)
    {
      el.removeEventListener("popupshowing", myForwardExtension.forward_fillMessagePaneContextMenu, false);
    }
  },

  // Pref observer

  register: function()
  {
    var prefService = Components.classes["@mozilla.org/preferences-service;1"].
                                 getService(Components.interfaces.nsIPrefService);
    this._forward_message_mode = prefService.getBranch("mail.forward_message_mode");  // Forward mode
    if (!("addObserver" in this._forward_message_mode))
      this._forward_message_mode.QueryInterface(Components.interfaces.nsIPrefBranch2); // Only necessary prior to Gecko 13
    this._forward_message_mode.addObserver("", this, false);

    this._pane_config_dynamic = prefService.getBranch("mail.pane_config.dynamic");  // Layout change
    if (!("addObserver" in this._pane_config_dynamic))
      this._pane_config_dynamic.QueryInterface(Components.interfaces.nsIPrefBranch2); // Only necessary prior to Gecko 13
    this._pane_config_dynamic.addObserver("", this, false);
  },

  unregister: function()
  {
    if(!this._forward_message_mode)
    {
      return;
    }
    this._forward_message_mode.removeObserver("", this);
    this._pane_config_dynamic.removeObserver("", this);
  },

  observe: function(aSubject, aTopic, aData)
  {
    if(aTopic !== "nsPref:changed")
    {
      return;
    }
    this.updateForwardButtonMenu();
  }
}

// Add command controller
top.controllers.appendController(myForwardExtension.forwardController);

// Add event listeners
window.addEventListener("load", myForwardExtension.installListeners, false);
window.addEventListener("unload", myForwardExtension.uninstallListeners, false);

// Watch for prefence changes
myForwardExtension.register();

})();
